@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -27,13 +27,13 @@
 #include <ctype.h>
 
 #include "typesx.h"
-#include "functionsx.h"
-
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
 #include "relationsx.h"
 
+#include "osmparser.h"
+
 #include "xmlparse.h"
 #include "tagging.h"
 
@@ -90,12 +90,13 @@ static double parse_length(way_t id,const char *k,const char *v);
 /* The XML tag processing function prototypes */
 
 //static int xmlDeclaration_function(const char *_tag_,int _type_,const char *version,const char *encoding);
-//static int osmType_function(const char *_tag_,int _type_);
+static int osmType_function(const char *_tag_,int _type_,const char *version);
 static int relationType_function(const char *_tag_,int _type_,const char *id);
 static int wayType_function(const char *_tag_,int _type_,const char *id);
 static int memberType_function(const char *_tag_,int _type_,const char *type,const char *ref,const char *role);
 static int ndType_function(const char *_tag_,int _type_,const char *ref);
 static int nodeType_function(const char *_tag_,int _type_,const char *id,const char *lat,const char *lon);
+static int changesetType_function(const char *_tag_,int _type_);
 static int tagType_function(const char *_tag_,int _type_,const char *k,const char *v);
 //static int boundType_function(const char *_tag_,int _type_);
 //static int boundsType_function(const char *_tag_,int _type_);
@@ -124,6 +125,13 @@ static xmltag tagType_tag=
                tagType_function,
                {NULL}};
 
+/*+ The changesetType type tag. +*/
+static xmltag changesetType_tag=
+              {"changeset",
+               0, {NULL},
+               changesetType_function,
+               {&tagType_tag,NULL}};
+
 /*+ The nodeType type tag. +*/
 static xmltag nodeType_tag=
               {"node",
@@ -162,9 +170,9 @@ static xmltag relationType_tag=
 /*+ The osmType type tag. +*/
 static xmltag osmType_tag=
               {"osm",
-               0, {NULL},
-               NULL,
-               {&boundsType_tag,&boundType_tag,&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
+               1, {"version"},
+               osmType_function,
+               {&boundsType_tag,&boundType_tag,&changesetType_tag,&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
 
 /*+ The xmlDeclaration type tag. +*/
 static xmltag xmlDeclaration_tag=
@@ -241,6 +249,24 @@ static int tagType_function(const char *_tag_,int _type_,const char *k,const cha
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the changesetType XSD type is seen
+
+  int changesetType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int changesetType_function(const char *_tag_,int _type_)
+{
+ current_tags=NULL;
+
+ return(0);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   The function that is called when the nodeType XSD type is seen
 
@@ -289,7 +315,7 @@ static int nodeType_function(const char *_tag_,int _type_,const char *id,const c
 
     process_node_tags(result,node_id,latitude,longitude);
 
-    DeleteTagList(current_tags);
+    DeleteTagList(current_tags); current_tags=NULL;
     DeleteTagList(result);
    }
 
@@ -454,7 +480,7 @@ static int wayType_function(const char *_tag_,int _type_,const char *id)
 
     process_way_tags(result,way_id);
 
-    DeleteTagList(current_tags);
+    DeleteTagList(current_tags); current_tags=NULL;
     DeleteTagList(result);
    }
 
@@ -509,7 +535,7 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
 
     process_relation_tags(result,relation_id);
 
-    DeleteTagList(current_tags);
+    DeleteTagList(current_tags); current_tags=NULL;
     DeleteTagList(result);
    }
 
@@ -525,12 +551,20 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
   const char *_tag_ Set to the name of the element tag that triggered this function call.
 
   int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *version The contents of the 'version' attribute (or NULL if not defined).
   ++++++++++++++++++++++++++++++++++++++*/
 
-//static int osmType_function(const char *_tag_,int _type_)
-//{
-// return(0);
-//}
+static int osmType_function(const char *_tag_,int _type_,const char *version)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    if(!version || strcmp(version,"0.6"))
+       XMLPARSE_MESSAGE(_tag_,"Invalid value for 'version' (only '0.6' accepted)");
+   }
+
+ return(0);
+}
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -628,6 +662,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
 
  for(i=0;i<tags->ntags;i++)
    {
+    int recognised=0;
     char *k=tags->k[i];
     char *v=tags->v[i];
 
@@ -640,6 +675,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Bicycle;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'bicycle' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -651,6 +687,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Foot;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'foot' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -662,6 +699,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Goods;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'goods' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -669,7 +707,10 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
       case 'h':
        if(!strcmp(k,"highway"))
           if(!strcmp(v,"mini_roundabout"))
+            {
              flags|=NODE_MINIRNDBT;
+             recognised=1; break;
+            }
 
        if(!strcmp(k,"horse"))
          {
@@ -677,6 +718,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Horse;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'horse' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"hgv"))
@@ -685,6 +727,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_HGV;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'hgv' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -696,6 +739,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Moped;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'moped' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorbike"))
@@ -704,6 +748,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Motorbike;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'motorbike' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorcar"))
@@ -712,6 +757,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Motorcar;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'motorcar' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -723,6 +769,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_PSV;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'psv' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -734,13 +781,17 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Wheelchair;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'wheelchair' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       default:
-       logerror("Node %"Pnode_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       break;
       }
+
+    if(!recognised)
+       logerror("Node %"Pnode_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
    }
 
  /* Create the node */
@@ -760,7 +811,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
 static void process_way_tags(TagList *tags,way_t id)
 {
  Way   way={0};
- int   oneway=0;
+ int   oneway=0,roundabout=0,area=0;
  char *name=NULL,*ref=NULL,*refname=NULL;
  int i;
 
@@ -789,11 +840,24 @@ static void process_way_tags(TagList *tags,way_t id)
 
  for(i=0;i<tags->ntags;i++)
    {
+    int recognised=0;
     char *k=tags->k[i];
     char *v=tags->v[i];
 
     switch(*k)
       {
+      case 'a':
+       if(!strcmp(k,"area"))
+         {
+          if(ISTRUE(v))
+             area=1;
+          else if(!ISFALSE(v))
+             logerror("Way %"Pway_t" has an unrecognised tag value 'area' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
+         }
+
+       break;
+
       case 'b':
        if(!strcmp(k,"bicycle"))
          {
@@ -801,6 +865,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Bicycle;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'bicycle' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"bicycleroute"))
@@ -809,6 +874,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_BicycleRoute;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"bridge"))
@@ -817,6 +883,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Bridge;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'bridge' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -828,6 +895,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Foot;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'foot' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"footroute"))
@@ -836,6 +904,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_FootRoute;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'footroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -847,13 +916,14 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Goods;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'goods' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'h':
        if(!strcmp(k,"highway"))
-          ;
+         {recognised=1; break;}
 
        if(!strcmp(k,"horse"))
          {
@@ -861,6 +931,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Horse;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'horse' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"hgv"))
@@ -869,25 +940,44 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_HGV;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'hgv' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'm':
-       if(!strcmp(k,"maxspeed"))
-          way.speed=kph_to_speed(parse_speed(id,k,v));
+       if(!strncmp(k,"max",3))
+         {
+          if(!strcmp(k+3,"speed"))
+            {
+             way.speed=kph_to_speed(parse_speed(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxweight"))
-          way.weight=tonnes_to_weight(parse_weight(id,k,v));
+          if(!strcmp(k+3,"weight"))
+            {
+             way.weight=tonnes_to_weight(parse_weight(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxheight"))
-          way.height=metres_to_height(parse_length(id,k,v));
+          if(!strcmp(k+3,"height"))
+            {
+             way.height=metres_to_height(parse_length(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxwidth"))
-          way.width=metres_to_height(parse_length(id,k,v));
+          if(!strcmp(k+3,"width"))
+            {
+             way.width=metres_to_height(parse_length(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxlength"))
-          way.length=metres_to_height(parse_length(id,k,v));
+          if(!strcmp(k+3,"length"))
+            {
+             way.length=metres_to_height(parse_length(id,k,v));
+             recognised=1; break;
+            }
+         }
 
        if(!strcmp(k,"moped"))
          {
@@ -895,6 +985,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Moped;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'moped' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorbike"))
@@ -903,6 +994,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Motorbike;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'motorbike' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorcar"))
@@ -911,6 +1003,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Motorcar;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'motorcar' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"multilane"))
@@ -919,13 +1012,17 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Multilane;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'multilane' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'n':
        if(!strcmp(k,"name"))
+         {
           name=v;
+          recognised=1; break;
+         }
 
        break;
 
@@ -938,6 +1035,7 @@ static void process_way_tags(TagList *tags,way_t id)
              oneway=-1;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'oneway' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -949,6 +1047,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Paved;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'paved' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"psv"))
@@ -957,13 +1056,26 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_PSV;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'psv' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'r':
        if(!strcmp(k,"ref"))
+         {
           ref=v;
+          recognised=1; break;
+         }
+
+       if(!strcmp(k,"roundabout"))
+         {
+          if(ISTRUE(v))
+             roundabout=1;
+          else if(!ISFALSE(v))
+             logerror("Way %"Pway_t" has an unrecognised tag value 'roundabout' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
+         }
 
        break;
 
@@ -974,6 +1086,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Tunnel;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'tunnel' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -985,13 +1098,17 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Wheelchair;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'wheelchair' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       default:
-       logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       break;
       }
+
+    if(!recognised)
+       logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
    }
 
  /* Create the way */
@@ -1009,6 +1126,9 @@ static void process_way_tags(TagList *tags,way_t id)
  if(oneway)
     way.type|=Way_OneWay;
 
+ if(roundabout)
+    way.type|=Way_Roundabout;
+
  if(ref && name)
    {
     refname=(char*)malloc(strlen(ref)+strlen(name)+4);
@@ -1032,11 +1152,11 @@ static void process_way_tags(TagList *tags,way_t id)
     node_t to  =way_nodes[i];
 
     if(oneway>0)
-       AppendSegment(segments,id,from,to,ONEWAY_1TO2);
+       AppendSegment(segments,id,from,to,area+ONEWAY_1TO2);
     else if(oneway<0)
-       AppendSegment(segments,id,from,to,ONEWAY_2TO1);
+       AppendSegment(segments,id,from,to,area+ONEWAY_2TO1);
     else
-       AppendSegment(segments,id,from,to,0);
+       AppendSegment(segments,id,from,to,area);
    }
 }
 
@@ -1061,6 +1181,7 @@ static void process_relation_tags(TagList *tags,relation_t id)
 
  for(i=0;i<tags->ntags;i++)
    {
+    int recognised=0;
     char *k=tags->k[i];
     char *v=tags->v[i];
 
@@ -1073,6 +1194,7 @@ static void process_relation_tags(TagList *tags,relation_t id)
              routes|=Transports_Bicycle;
           else if(!ISFALSE(v))
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -1081,11 +1203,13 @@ static void process_relation_tags(TagList *tags,relation_t id)
        if(!strcmp(k,"except"))
          {
           for(i=1;i<Transport_Count;i++)
-             if(!strstr(v,TransportName(i)))
+             if(strstr(v,TransportName(i)))
                 except|=TRANSPORTS(i);
 
           if(except==Transports_None)
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'except' = '%s' (after tagging rules); ignoring it.\n",id,v);
+
+          recognised=1; break;
          }
 
        break;
@@ -1097,6 +1221,7 @@ static void process_relation_tags(TagList *tags,relation_t id)
              routes|=Transports_Foot;
           else if(!ISFALSE(v))
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'footroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -1114,19 +1239,30 @@ static void process_relation_tags(TagList *tags,relation_t id)
 
           if(restriction==TurnRestrict_None)
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'restriction' = '%s' (after tagging rules); ignoring it.\n",id,v);
+
+          recognised=1; break;
          }
 
        break;
 
       case 't':
        if(!strcmp(k,"type"))
+         {
           if(!strcmp(v,"restriction"))
              relation_turn_restriction=1;
+
+          /* Don't log an error for relations of types that we don't handle - there are so many */
+          recognised=1; break;
+         }
+
        break;
 
       default:
-       logerror("Relation %"Prelation_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       break;
       }
+
+    if(!recognised)
+       logerror("Relation %"Prelation_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
    }
 
  /* Create the route relation (must store all relations that have ways or
