@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -25,12 +25,16 @@
 #include <string.h>
 
 #include "types.h"
-#include "visualiser.h"
 #include "nodes.h"
 #include "segments.h"
 #include "ways.h"
 #include "relations.h"
 
+#include "visualiser.h"
+
+
+/*+ The maximum number of segments per node (used to size temporary storage). +*/
+#define MAX_SEG_PER_NODE 32
 
 /* Limit types */
 
@@ -57,6 +61,8 @@ static double LonMin;
 static double LonMax;
 
 static int limit_type=0;
+static Highway highways=Way_Count;
+static Transports transports=Transports_None;
 
 /* Local functions */
 
@@ -64,6 +70,8 @@ static void find_all_nodes(Nodes *nodes,callback_t callback);
 static void output_junctions(index_t node,double latitude,double longitude);
 static void output_super(index_t node,double latitude,double longitude);
 static void output_oneway(index_t node,double latitude,double longitude);
+static void output_highway(index_t node,double latitude,double longitude);
+static void output_transport(index_t node,double latitude,double longitude);
 static void output_turnrestriction(index_t node,double latitude,double longitude);
 static void output_limits(index_t node,double latitude,double longitude);
 
@@ -310,6 +318,182 @@ static void output_oneway(index_t node,double latitude,double longitude)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Output the data for segments of a particular highway type.
+
+  Nodes *nodes The set of nodes to use.
+
+  Segments *segments The set of segments to use.
+
+  Ways *ways The set of ways to use.
+
+  Relations *relations The set of relations to use.
+
+  double latmin The minimum latitude.
+
+  double latmax The maximum latitude.
+
+  double lonmin The minimum longitude.
+
+  double lonmax The maximum longitude.
+
+  Highway highway The type of highway.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void OutputHighway(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,double latmin,double latmax,double lonmin,double lonmax,Highway highway)
+{
+ /* Use local variables so that the callback doesn't need to pass them backwards and forwards */
+
+ OSMNodes=nodes;
+ OSMSegments=segments;
+ OSMWays=ways;
+ OSMRelations=relations;
+
+ LatMin=latmin;
+ LatMax=latmax;
+ LonMin=lonmin;
+ LonMax=lonmax;
+
+ /* Iterate through the nodes and process them */
+
+ highways=highway;
+
+ find_all_nodes(nodes,(callback_t)output_highway);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Process a single node and all connected one-way segments (called as a callback).
+
+  index_t node The node to output.
+
+  double latitude The latitude of the node.
+
+  double longitude The longitude of the node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void output_highway(index_t node,double latitude,double longitude)
+{
+ Node *nodep=LookupNode(OSMNodes,node,1);
+ Segment *segment;
+
+ segment=FirstSegment(OSMSegments,nodep,1);
+
+ do
+   {
+    if(IsNormalSegment(segment))
+      {
+       index_t othernode=OtherNode(segment,node);
+
+       if(node>othernode)
+         {
+          Way *way=LookupWay(OSMWays,segment->way,1);
+
+          if(way->type==highways)
+            {
+             double lat,lon;
+
+             GetLatLong(OSMNodes,othernode,&lat,&lon);
+
+             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+            }
+         }
+      }
+
+    segment=NextSegment(OSMSegments,segment,node);
+   }
+ while(segment);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Output the data for segments allowed for a paticular type of traffic.
+
+  Nodes *nodes The set of nodes to use.
+
+  Segments *segments The set of segments to use.
+
+  Ways *ways The set of ways to use.
+
+  Relations *relations The set of relations to use.
+
+  double latmin The minimum latitude.
+
+  double latmax The maximum latitude.
+
+  double lonmin The minimum longitude.
+
+  double lonmax The maximum longitude.
+
+  Transport transport The type of transport.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void OutputTransport(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,double latmin,double latmax,double lonmin,double lonmax,Transport transport)
+{
+ /* Use local variables so that the callback doesn't need to pass them backwards and forwards */
+
+ OSMNodes=nodes;
+ OSMSegments=segments;
+ OSMWays=ways;
+ OSMRelations=relations;
+
+ LatMin=latmin;
+ LatMax=latmax;
+ LonMin=lonmin;
+ LonMax=lonmax;
+
+ /* Iterate through the nodes and process them */
+
+ transports=TRANSPORTS(transport);
+
+ find_all_nodes(nodes,(callback_t)output_transport);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Process a single node and all connected one-way segments (called as a callback).
+
+  index_t node The node to output.
+
+  double latitude The latitude of the node.
+
+  double longitude The longitude of the node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void output_transport(index_t node,double latitude,double longitude)
+{
+ Node *nodep=LookupNode(OSMNodes,node,1);
+ Segment *segment;
+
+ segment=FirstSegment(OSMSegments,nodep,1);
+
+ do
+   {
+    if(IsNormalSegment(segment))
+      {
+       index_t othernode=OtherNode(segment,node);
+
+       if(node>othernode)
+         {
+          Way *way=LookupWay(OSMWays,segment->way,1);
+
+          if(way->allow&transports)
+            {
+             double lat,lon;
+
+             GetLatLong(OSMNodes,othernode,&lat,&lon);
+
+             printf("%.6f %.6f %.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude),radians_to_degrees(lat),radians_to_degrees(lon));
+            }
+         }
+      }
+
+    segment=NextSegment(OSMSegments,segment,node);
+   }
+ while(segment);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Output the data for turn restrictions.
 
@@ -620,11 +804,9 @@ void OutputLengthLimits(Nodes *nodes,Segments *segments,Ways *ways,Relations *re
 
 static void output_limits(index_t node,double latitude,double longitude)
 {
-#define MAX_STORED 32
  Node *nodep=LookupNode(OSMNodes,node,1);
- Segment *segment,*segments[MAX_STORED];
- Way *ways[MAX_STORED];
- int limits[MAX_STORED];
+ Segment *segment,segments[MAX_SEG_PER_NODE];
+ int limits[MAX_SEG_PER_NODE];
  int count=0;
  int i,j,same=0;
 
@@ -632,21 +814,22 @@ static void output_limits(index_t node,double latitude,double longitude)
 
  do
    {
-    if(IsNormalSegment(segment) && count<MAX_STORED)
+    if(IsNormalSegment(segment) && count<MAX_SEG_PER_NODE)
       {
-       ways    [count]=LookupWay(OSMWays,segment->way,1);
-       segments[count]=segment;
+       Way *way=LookupWay(OSMWays,segment->way,1);
+
+       segments[count]=*segment;
 
        switch(limit_type)
          {
-         case SPEED_LIMIT:  limits[count]=ways[count]->speed;  break;
-         case WEIGHT_LIMIT: limits[count]=ways[count]->weight; break;
-         case HEIGHT_LIMIT: limits[count]=ways[count]->height; break;
-         case WIDTH_LIMIT:  limits[count]=ways[count]->width;  break;
-         case LENGTH_LIMIT: limits[count]=ways[count]->length; break;
+         case SPEED_LIMIT:  limits[count]=way->speed;  break;
+         case WEIGHT_LIMIT: limits[count]=way->weight; break;
+         case HEIGHT_LIMIT: limits[count]=way->height; break;
+         case WIDTH_LIMIT:  limits[count]=way->width;  break;
+         case LENGTH_LIMIT: limits[count]=way->length; break;
          }
 
-       if(limits[count] || ways[count]->type<Way_Track)
+       if(limits[count] || HIGHWAY(way->type)<Way_Track)
           count++;
       }
 
@@ -659,7 +842,7 @@ static void output_limits(index_t node,double latitude,double longitude)
  if(count==1)
     return;
 
- /* Nodes with all segments the same limit is not interesting */
+ /* Nodes with all segments the same limit are not interesting */
 
  same=0;
  for(j=0;j<count;j++)
@@ -669,7 +852,7 @@ static void output_limits(index_t node,double latitude,double longitude)
  if(same==count)
     return;
 
- /* Display the interesting speed limits */
+ /* Display the interesting limits */
 
  printf("%.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude));
 
@@ -684,7 +867,7 @@ static void output_limits(index_t node,double latitude,double longitude)
       {
        double lat,lon;
 
-       GetLatLong(OSMNodes,OtherNode(segments[i],node),&lat,&lon);
+       GetLatLong(OSMNodes,OtherNode(&segments[i],node),&lat,&lon);
 
        switch(limit_type)
          {
